<!--###############################################################################
## OpenConference project
##
## Copyright (C) 2009 Antonio Falcón Aragón  antonio.falconaragon@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################-->

<?php

class Posts extends Controller{
	//Variable protegida
	protected $_template;

	//Constructor
	public function __construct(){
		parent::Controller();
		
		//Comprobamos si el usuario logueado es el administrador
		$this->access_library->check_access();

		//cargamos los modulos, librerias y todos los archivos necesarios
		$this->load->module_model('admin','posts_model','posts');	
	}

	//Métodos públicos
	public function index(){
		$data['posts']=$this->posts->listar_posts();

		$this->_template['pagina']= 'posts/listar';

		$this->system_library->load($this->_template['pagina'], $data, TRUE);
	}

	public function crear(){
		$this->form_validation->set_rules('titulo', 'Titulo', 'required|max_length[200]');
		$this->form_validation->set_rules('excerpt', 'Excerpt', 'required|prep_for_form');
		$this->form_validation->set_rules('contenido', 'Contenido', 'prep_for_form');
		$this->form_validation->set_rules('estado', 'Estado', 'required|xss_clean');
		$this->form_validation->set_rules('permitir_comentarios', 'Permitir_comentarios', 'numeric');
		$this->form_validation->set_rules('sticky', 'Sticky', 'numeric');

		$this->form_validation->set_error_delimiters('', '<br />');

		$data['settings']['permitir_comentarios'] = 'checked';

		if ($this->form_validation->run() == TRUE)
		{
			$this->posts->crear_post();
	
			$this->session->set_flashdata('message', 'Entrada creada correctamente');

			redirect('admin/posts', 'refresh');
		}

		$this->_template['pagina'] = 'posts/crear';
		
		$this->system_library->load($this->_template['pagina'], $data, TRUE);

	}	

	public function editar($id=NULL){
		if($id==NULL)
			$id=$this->input->post('id');
		
		$this->form_validation->set_rules('titulo', 'Titulo', 'required|max_length[200]');
		$this->form_validation->set_rules('excerpt', 'Excerpt', 'required|prep_for_form');
		$this->form_validation->set_rules('contenido', 'Contenido', 'prep_for_form');
		$this->form_validation->set_rules('estado', 'Estado', 'required|xss_clean');
		$this->form_validation->set_rules('permitir_comentarios', 'Permitir_comentarios', 'numeric');
		$this->form_validation->set_rules('sticky', 'Sticky', 'numeric');
		
		$this->form_validation->set_error_delimiters('', '<br />');

		$data['post'] = $this->posts->listar_post($id);
		$data['settings']['permitir_comentarios'] = ($data['post']['permitir_comentarios'] == 1) ? 'checked' : '';
		$data['settings']['sticky'] = ($data['post']['sticky'] == 1) ? 'checked' : '';

		if ($this->form_validation->run() == TRUE)
		{
			$this->posts->editar_post($id);

			$this->session->set_flashdata('message', 'Entrada editada satisfactoriamente');

			redirect('admin/posts', 'refresh');
		}
		
		$this->_template['pagina'] = 'posts/editar';
			
		$this->system_library->load($this->_template['pagina'], $data, TRUE);
	}

	public function eliminar($post_id){
		$this->posts->eliminar_post($post_id);
		
		$this->session->set_flashdata('message', 'Entrada eliminada satisfactoriamente');

		redirect('admin/posts', 'refresh');
	}
}

?>
