<!--###############################################################################
## OpenConference project
##
## Copyright (C) 2009 Antonio Falcón Aragón  antonio.falconaragon@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################-->

<?php

class Cfp_model extends Model
{
	// Variable protegida
	protected $_table;
	
	// Constructor
	public function __construct()
	{
		parent::Model();
			
		$this->_table = $this->config->item('database_tables');
	}

	// Métodos públicos
	public function listar_cfps()
	{
		$this->db->select('id, fecha_envio, nombre, tema, estado, revisiones');
			
		$query = $this->db->get($this->_table['cfp']);
			
		if ($query->num_rows() > 0)
		{
			return $query->result_array();
		}
	}
	
	public function listar_cfp($id)
	{
		$this->db->select('id, fecha_envio, nombre, tema, estado, descripcion');
		$this->db->where('id', $id);
			
		$query = $this->db->get($this->_table['cfp'], 1);
			
		if ($query->num_rows() == 1)
		{
			return $query->row_array();
		}
	}

	public function revisar_cfp($id)
	{
		$data = array(
			'estado' => $this->input->post('estado'),
		      );
			
		$this->db->where('id', $id);
		$this->db->update($this->_table['cfp'], $data);

		$this->actualizar_revisiones($id);

		$datos = array(
			'paper_id' => $id,
			'valoracion' => $this->input->post('valoracion'),
			'resumen' => $this->input->post('resumen')
		      );

		$this->db->insert($this->_table['revision'], $datos);
	}

	public function eliminar_cfp($id)
	{
		$this->db->where('id', $id);
			
		$this->db->delete($this->_table['cfp']);

		$this->db->where('paper_id', $id);
	
		$this->db->delete($this->_table['revision']);
	}

	public function ver_cfp($id)
	{
		$this->db->select('paper_id, valoracion, resumen');
		$this->db->where('paper_id', $id);

		$query = $this->db->get($this->_table['revision']);
			
		if ($query->num_rows() > 0)
		{
			return $query->result_array();
		}
	}

	public function estado_revision($id)
	{
		$data = array(
			'estado' => 'revision'
		      );

		$this->db->where('id',$id);
		$this->db->update($this->_table['cfp'],$data);

	}

	public function estado_revisado($id)
	{
		$data = array(
			'estado' => 'revisado'
		      );

		$this->db->where('id',$id);
		$this->db->update($this->_table['cfp'],$data);

	}
	
	public function actualizar_revisiones($id)
	{
		$this->db->where('id', $id);
		$this->db->set('revisiones', 'revisiones+1', FALSE);
		$this->db->update($this->_table['cfp']); 

	}

	public function ver_revisiones($id)
	{
		$this->db->select('revisiones');
		$this->db->where('id', $id);
			
		$query = $this->db->get($this->_table['cfp'], 1);
			
		if ($query->num_rows() == 1)
		{
			return $query->row_array();
		}
	}

	public function estado_terminado($id)
	{
		$data = array(
			'estado' => 'terminado'
		      );

		$this->db->where('id',$id);
		$this->db->update($this->_table['cfp'],$data);
	}
}
?>
