<!--###############################################################################
## OpenConference project
##
## Copyright (C) 2009 Antonio Falcón Aragón  antonio.falconaragon@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################-->

<?php

class Usuario_model extends Model
{
	// Variable protegida
	protected $_table;
	
	// Constructor
	public function __construct()
	{
		parent::Model();
			
		$this->_table = $this->config->item('database_tables');
	}
	
	// Métodos públicos
	public function listar_usuario($id)
	{
		$this->db->select('id, username, email, nombre, apellidos, website, display_name, registrado, nivel');
		$this->db->where('id', $id);
			
		$query = $this->db->get($this->_table['usuarios'], 1);
			
		if ($query->num_rows() == 1)
		{
			return $query->row_array();
		}
	}
	
	public function conseguir_usuario_por_nick($nickname)
	{
		$this->db->select('id, username, email, nombre, apellidos, website, display_name, registrado, nivel');
		$this->db->where('username', $nickname);
			
		$query = $this->db->get($this->_table['usuarios'], 1);
			
		if ($query->num_rows() == 1)
		{
			return $query->row_array();
		}
	}
	
	public function comprueba_usuario($username, $password)
	{
		$this->db->select('id, username, nivel');
		$this->db->where('username', $username);
		$this->db->where('password', dohash($password, 'md5'));
		$this->db->where('nivel', 'administrador');
		$this->db->where('estado', '1');
			
		$query = $this->db->get($this->_table['usuarios'], 1);
			
		if ($query->num_rows() == 1)
		{
			$row = $query->row_array();
			
			return $row['id'];
		}
		else
		{
			return FALSE;
		}
	}

	public function verificar_usuario($username, $password)
	{
		$this->db->select('id, username, nivel, secret_key');
		$this->db->where('username', $username);
		$this->db->where('password', dohash($password, 'md5'));
		$this->db->where('estado', '1');
			
		$query = $this->db->get($this->_table['usuarios'], 1);
			
		if ($query->num_rows() == 1)
		{
			$row = $query->row_array();

			$data = array
						(
							'user_id' => $row['id'],
							'username' => $row['username'],
							'nivel' => $row['nivel'],
							'logged_in' => TRUE
						);
				
			$this->session->set_userdata($data);
			
			$this->clean_secret_key($row['id']);
		}
		else
		{
			$this->session->set_flashdata('error', 'Username no valido');
		}
	}

	public function actualizar_ultimo_acceso()
	{
		$data = array
					(
						'ultimo_acceso' => date('Y-m-d H:i:s')
					);

		$this->db->where('id', $this->session->userdata('user_id'));
		$this->db->update($this->_table['usuarios'], $data);
	}

	public function logout()
	{
		$data = array
					(
						'user_id' => 0,
						'username' => 0,
						'nivel' => 0,
						'logged_in' => FALSE
					);

		$this->session->sess_destroy();
		$this->session->unset_userdata($data);
	}
	
	public function comprobar_validacion($data)
	{
		$this->db->select('id');
		$this->db->where($data);
		
		$query = $this->db->get($this->_table['usuarios']);
		
		if ($query->num_rows() == 1)
		{
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
	
	public function check_secret_key($key, $email)
	{
		$this->db->select('id');
		$this->db->where('secret_key', $key);
		$this->db->where('email', $email);
		
		$query = $this->db->get($this->_table['usuarios'], 1);
			
		if ($query->num_rows() == 1)
		{
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}

	public function crear_usuario()
	{
		$secret_key = dohash(uniqid() + time(), 'md5');
		
		$data = array
					(
						'username' => $this->input->post('username'),
						'password' => dohash($this->input->post('password'), 'md5'),
						'secret_key' => $secret_key,
						'display_name' => $this->input->post('display_name'),
						'email' => $this->input->post('email'),
						'nombre' => $this->input->post('nombre'),
						'apellidos' => $this->input->post('apellidos'),
						'website' => $this->input->post('website'),
						'registrado' => date('Y-m-d H:i:s')
						);

		$this->db->insert($this->_table['usuarios'], $data);
		
		$this->send_secret_key($this->input->post('email'), $secret_key, 'Activacion_cuenta');
	}

	public function editar_perfil($id)
	{
		$data = array
					(
						'display_name' => $this->input->post('display_name'),
						'nombre' => $this->input->post('nombre'),
						'apellidos' => $this->input->post('apellidos'),
						'email' => $this->input->post('email'),
						'website' => $this->input->post('website')
					);
			
		if ($this->input->post('password') != "")
		{
			$data['password'] = dohash($this->input->post('password'), 'md5');
		}
			
		$this->db->where('id', $id);
		$this->db->update($this->_table['usuarios'], $data);
	}
	
	public function password_olvidado()
	{
		$key = dohash(uniqid() + time(), 'md5');
		
		$data = array
					(
						'secret_key' => $key
					);
					
		$this->db->where('username', $this->input->post('username'));
		$this->db->where('email', $this->input->post('email'));
		$this->db->update($this->_table['usuarios'], $data);
		
		$this->send_secret_key($this->input->post('email'), $key, 'Password_olvidado');
	}
	
	public function activa_cuenta($key, $email)
	{
		$data = array
					(
						'secret_key' => '',
						'estado' => '1'
					);

		$this->db->where('secret_key', $key);
		$this->db->where('email', $email);
		$this->db->update($this->_table['usuarios'], $data);
	}
	
	public function reestablecer_password($key, $email)
	{
		$nuevo_password = random_string('alnum', 8);
		
		$data = array
					(
						'password' => dohash($nuevo_password, 'md5'),
						'secret_key' => ''
					);

		$this->db->where('secret_key', $key);
		$this->db->where('email', $email);
		$this->db->update($this->_table['usuarios'], $data);
		
		$this->send_new_password($email, $nuevo_password);
	}
	
	// Métodos protegidos	
	protected function send_secret_key($email, $key, $type = 'Activacion_cuenta')
	{
		$this->load->library('email');
		
		if ($type == 'Activacion_cuenta')
		{
			$activation_url = account_activation_url($email, $key);
			
			$email_subject = "Activacion de Cuenta";;
			$email_body = "<p>Hola,</p>
			<p>Tu cuenta se ha creado correctamente, pero antes de poder usarla tienes que activarla.</p>
			<p>Tu puedes hacerlo haciendo click en el siguiente enlace:</p>
			<p><a href='".$activation_url."'>".$activation_url."</a></p>
			<p>Si al hacer click en el enlace anterior no funciona, copia y pega la URL en<br />
			una nueva ventana del navegador.</p>
			<p>Por favor, ignora este mensaje si no vas a registrarte en la Web.</p>
			<p>Este mensaje es generado autom&aacute;ticamente, no responda a este mensaje.</p>
			<p>Atentamente,<br />The OpenConference Team</p>";
		}
		else if ($type == 'Password_olvidado')
		{
			$activation_url = forgotten_password_url($email, $key);
			
			$email_subject = "Recordatorio de Password";
			$email_body = "<p>Hola,</p>
			<p>Hemos recibido una solicitud de recordatorio de password desde tu cuenta de usuario.</p>
			<p>Para iniciar el proceso, por favor, haz click en el siguiente enlace:</p>
			<p><a href='".$activation_url."'>".$activation_url."</a></p>
			<p>ISi al hacer click en el enlace anterior no funciona, copia y pega la URL en<br />
			una nueva ventana del navegador.</p>
			<p>Por favor, ignora este mensaje si no has solicitado un recordatorio de password.</p>
			<p>Este mensaje es generado autom&aacute;ticamente, no responda a este mensaje.</p>
			<p>Atentamente,<br />The OpenConference Team</p>";
		}
		
		$this->email->from($this->system_library->settings['admin_email'], $this->system_library->settings['titulo_pagina']);
		$this->email->to($email);
		$this->email->subject($email_subject);
		$this->email->message($email_body);
				
		$this->email->send();	
	}
	
	protected function send_new_password($email, $password)
	{
		$this->load->library('email');
		
		$login_url = site_url('usuario/login');
		
		$this->email->from($this->system_library->settings['admin_email'], $this->system_library->settings['titulo_pagina']);
		$this->email->to($email);
				
		$this->email->subject("Nuevo Password");
		$this->email->message("<p>Hola,</p>
		<p>Tu password ha sido reestablecido satisfactoriamente.</p>
		<p>Tu nuevo password es: <strong>%s</strong></p>
		<p>Tu puedes ahora <a href='%s'>acceder</a> con el password ofrecido anteriormente.</p>
		<p>Este mensaje es generado autom&aacute;ticamente, no responda a este mensaje.</p>
		<p>Atentamente,<br />The OpenConference Team</p>", array($password, $login_url));
				
		$this->email->send();
	}
	
	protected function clean_secret_key($user_id)
	{
		$this->db->select('secret_key');
		$this->db->where('id', $user_id);
		
		$query = $this->db->get($this->_table['usuarios'], 1);
		
		if ($query->num_rows() == 1)
		{
			$data = array
					(
						'secret_key' => ''
					);
					
			$this->db->where('id', $user_id);
			$this->db->update($this->_table['usuarios'], $data);
		}
	}
}
?>
